// Print Gray codes verbosely
// Michael Ernst <mernst@alum.mit.edu>
// August 19, 1999

#include <iostream>
#include <string>
#include <strstream>
// To disable the assertions (and dramatically improve runtime),
// compile with the "-DNDEBUG" compiler flag.  (Don't forget -O in that case.)
#include <assert.h>
#include "gray-codes.h"

// Example compilation lines:
//  For debugging:   g++ -Wall -g -o gray-codes-postprocess gray-codes-postprocess.cpp
//  It probably doesn't make sense to compile for speed; I want the extra
//  checking in this program.
//  // For speed:       g++ -Wall -DNDEBUG -O3 -o gray-codes-postprocess gray-codes-postprocess.cpp

// Run this on the output of the "gray-codes" program, with the leading short
// gray codes stripped off.
// Sample command line:
//   gray-codes-postprocess < gray-codes-6-output-prefix > gray-codes-6-prefix-postprocessed

int main() {
  // The reason I may want to use graph_constraint == no_bidirectional_edges is
  // so maintain_induced is true.  I need to fix this so that that can be set
  // separately!
  if ((graph_constraint != no_constraint)
      && (graph_constraint != no_bidirectional_edges)){
    cerr << "Run postprocess only with graph_constraint == no_constraint\n";
    abort();
  }

  cout << "Checking Gray codes for " << BITS << " bits.\n";

  // I couldn't get ostringstream to work here.  Why?
  ostrstream os;

  while (!cin.eof()) {
    string s;
    cin >> s;
    if (s != "bits:") {
      getline(cin, s);
      continue;
    }
    getline(cin, s);
    Gray_Code gc(s);
    cout << "===========================================================================\n";
    gc.print_induced();
    cout << "\n";
    // Reinstate this later; it depends on induced[][] being maintained.
    // cout << "Girth " << gc.girth() << "\n";
    gc.print_gray_code_short();
    gc.print_gray_code();
    if (gc.num_trans < STATES-1) {
      cout << STATES-1 - gc.num_trans << " missing codewords:\n";
      for (int i=0; i<STATES; i++) {
        if (! gc.reached[i]) {
          cout << "  ";
          gc.print_binary(i);
          cout << "\n";
        }
      }
    }
  }
}
 
